Option Explicit

'isDate(Date)

Public bMACRO_STARTED As Boolean

REM --------------------------------------------------------------------------------------------------------------------------------
REM	Die Funktion prft, ob das gestartete Makro noch in Verarbeitung ist und gibt den Rckgabewert, sowie bei positivem Ergebnis, auch einen Warnhinweis aus.
REM	Am Anfang des Funktionsaufrufes kann dann z.B. mit "If isMacroStarted() Then Exit Sub", effektiv eine weitere Ausfhrung des Makros verhindert werden.
REM	Bemerkung: Fr die Funktion wird die Deklaration der globalen Variable "bMACRO_STARTED", sowie auch das Zurcksetzen dieser am Ende des Makros vorausgesetzt!
Public Function isMacroStarted(optional ByVal pbCompareToDirectExit As Boolean) As Boolean
'	pbCompareToDirectExit > eine Prfbedingung, mit der das Makro bei positivem Ergebnis auch ohne Warnhinweis beendet werden kann

Const cstThisSub = "validation.isMacroStarted"
	On Local Error GoTo NOTEXPECTED

	If Not isMissing(pbCompareToDirectExit) And pbCompareToDirectExit Then
		isMacroStarted() = True
		Exit Function
	End If

	If bMACRO_STARTED Then
		isMacroStarted() = True
		GoTo FUNCTIONUSED
	Else
		bMACRO_STARTED = True
	End If
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
	Exit Function
FUNCTIONUSED:
	traceError(cstThisSub, Erl, cstWARNING, ERRFUNCTIONUSED, True)
End Function 'isMacroStarted V1.0

REM --------------------------------------------------------------------------------------------------------------------------------
REM	Die Funktion testet auf eine bestehende Internetverbindung, indem versucht wird, den Inhalt eines Standard-URLs zu extrahieren.
Public Function isWebConnected() As Boolean
'	Rckgabewert ist True, wenn die Fehlermeldung "Gerte-E/A-Fehler." nicht auftritt.

'	>>>TODO: Umbau der Funktion auf isFileExists("https://de.wikipedia.org/") und ... "https://www.startpage.com/"

Const cstThisSub = "validation.isWebConnected"
	On Local Error GoTo WEBNOTCONNECTED
	
	Dim sTextline As String
	Dim sText As String
	Dim sWebURL As String
	Dim iNumber As Integer

	sWebURL = convertToURL("https://www.wikipedia.de")
	sText = ""

	iNumber = Freefile
	Open sWebUrl For Input As iNumber
	While Not EOF(iNumber)
		Line Input #iNumber, sTextline
		If sTextline <> "" Then sText = sText & sTextline & Chr(10)
	Wend

	Close #iNumber
	isWebConnected = True
	Exit Function

WEBNOTCONNECTED:
	If Err = 57 Then 'Code: "Gerte-E/A-Fehler."
		isWebConnected() = False
		traceError(cstThisSub, Erl, cstWARNING, ERRWEBNOTCONNECTED, True, sWebURL)

'		>>>TODO: "Auftrag wurde zur Warteschlange hinzugefgt."
	End If
End Function 'isWebConnected V1.0.1

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion validiert mittels Pattern-Matching eine eMail-Adresse laut Request For Comments (RFC) 5321/5322 und Spezifikation des Internet Message Format (IMF).
Public Function isMailValid(ByVal psMailAddress As String) As Boolean
'	psMailAddress > die eMail-Adresse, welche validiert werden soll

'	Aufbau einer eMail-Adresse an einem Beispiel:

'	erika.mustermann@musterstadt.muster.de
'	xxxxx.xxxxxxxxxx						lokale Adresse (local-part)
'					 xxxxxxxxxxx.xxxxxx.xx	Provider/Host (domain-part); bestehend aus mindestens zwei Teilabschnitten ("Labels")
'										xx	1. Ebene, Top-Level-Domain(TLD)
'								 xxxxxx		2. Ebene, 2nd-Level-Domain
'					 xxxxxxxxxxx			3. Ebene, 3rd-Level-Domain (usw. ...)

'	>>>TODO: Bei ungltigen Zeichen oder -kombinationen, werden diese prezisiert ber eine Meldung am Bildschirm ausgegeben.

Const cstThisSub = "validation.isMailValid"
	On Local Error GoTo NOTEXPECTED

	Dim sChars As String
	Dim sCharsQuotation As String
	Dim sSearchStandard As String
	Dim sSearchQuotations As String
	Dim sSearchComments As String
	Dim sSearchLabels As String
	Dim n As Integer

'	Definition eines Standard-Rckgabewertes
	isMailValid() = False

'	If Not isArgumentChecked(psMailAddress, varString) Then Exit Function

'	berprfung der eMail-Gesamtlnge
	If Len(psMailAddress) > 254 Then GoTo INVALIDMAILADDRESS
'	berprfung der Lnge des local-parts
	If Len(RTrimFromChars(psMailAddress, "@")) > 64 Then GoTo INVALIDMAILADDRESS

'	Standard-Zeichenauswahl des local-parts (inkl. Maskierungen mit Backslash)
	sChars = "[A-Za-z0-9\!\#\$\%\&\'\*\+\-\/\=\?\^\_\`\{\|\}\~]"
'	exklusive Zeichenauswahl innerhalb doppelter Anfhrungszeichen
	sCharsQuotation = "[ \""\(\)\,\:\;\<\>\@\[\\\]]"

'	Richtlinien des local-parts:
'	- gltige Standardzeichen sind [A-Za-z0-9.!#$%&'*+-/=?^_`{|}~]
'	- die Gesamtlnge betrgt 1 bis 64 Zeichen
'	- der local-part (oder ein mittels Punkten getrennter Teilabschnitt) knnen in doppelte Anfhrungszeichen gesetzt werden,
'	  z.B. "muster"@beispiel.org oder Max."Muster".Musterstrasse@beispiel.org
'	- gltige, zustzliche Zeichen innerhalb dieser Anfhrungszeichen sind [" (),:;<>@[\]],
'	  Backslash und weitere Anfhrungszeichen sind darin zu maskieren
'	- innerhalb von runden Klammern sind am Anfang oder Ende auch Kommentare erlaubt: z.B. Muster(comment)@beispiel.org oder (comment)Muster@beispiel.org

'	Such-Algorithmus:
'		x Standardzeichen und x folgende, durch "." abgetrennte Standardzeichen, welche als Block beliebig oft vorkommen knnen
	sSearchStandard	= "(" & sChars & "{1,64}(\." & sChars & "{1,62})*)"

'	Such-Algorithmus:
'		x Standardzeichen (keinmal oder beliebig oft) und als Trennzeichen mit "." abschliessend
'		x Standard- oder Sonderzeichen, eingeschlossen von doppelten Anfhrungszeichen
'		x Standardzeichen (keinmal oder beliebig oft) und als Trennzeichen mit "." beginnend
	sSearchQuotations = "((" & sChars & "{1,62}\.)*""(" & sChars & "|" & sCharsQuotation & "){1,62}""(\." & sChars & "{1,62})*)"

'	Such-Algorithmus:
'		x Standardzeichen, eingeschlossen in Klammern; gefolgt von x Standardzeichen oder...
'		x Standardzeichen vom Anfang des Textes; gefolgt von x Standardzeichen, eingeschlossen in Klammern
	sSearchComments	= "(\(" & sChars & "{1,61}\)" & sChars & "{1,61})|(" & sChars & "{1,61}\(" & sChars & "{1,61}\))"

'	Richtlinien des domain-parts:
'	- gltige ASCII-Zeichen sind [A-Za-z0-9-], wobei "-" nicht als erstes oder letztes Zeichen vorkommen darf
'	- die Gesamtlnge betrgt jeweils 1 bis 63 Zeichen

'	Such-Algorithmus:
'		x Zeichen aus [A-Za-z0-9] oder "-" (solang kein "@" oder "." vorangeht; und solang kein "." oder das Textende folgt)
	sSearchLabels = "([A-Za-z0-9]|((?<!(\.|@))[\-](?!(\.|$)))){1,63}"

'	Zusammenstellung des regulren Ausdrucks und berprfung der eMail-Adresse
	If psMailAddress <> searchByRegExpressions(psMailAddress, _
		"^(" & sSearchStandard & "|" & sSearchQuotations & "|" & sSearchComments & ")@(" & sSearchLabels & "(\." & sSearchLabels & ")+)$") Then GoTo INVALIDMAILADDRESS

	isMailValid() = True
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
	Exit Function
INVALIDMAILADDRESS:
	traceError(cstThisSub, Erl, cstWARNING, ERRINVALIDMAILADDRESS, True, psMailAddress)
End Function 'isMailValid V1.7

REM --------------------------------------------------------------------------------------------------------------------------------
REM in Bearbeitung...
REM Die Funktion validiert mittels Pattern-Matching einen beliebigen Namen.
Public Function isName(ByVal psName As String) As Boolean
'	sName > ein Name, welcher validiert werden soll

Const cstThisSub = "validation.isName"
	On Local Error GoTo NOTEXPECTED

'	Definition eines Standard-Rckgabewertes
	isName() = False

	If Not isArgumentChecked(psName, varString) Then Exit Function

'	Such-Algorithmus:
'		x Buchstaben oder bestimmte definierte, in Namen auftretende Sonderzeichen, von Anfang bis Ende des Textes
	If psName = searchByRegExpressions(psName, "^([:alpha:]|[ '`])+$") Then isName() = True
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isName V1.0

REM --------------------------------------------------------------------------------------------------------------------------------
REM	Die Funktion berprft, ob eine bergebene Jahreszahl ein Schaltjahr darstellt.
Public Function isLeapYear(ByVal pnYear As Integer) as Boolean
'	pnYear > eine Jahreszahl, die berprft werden soll

Const cstThisSub = "validation.isLeapYear"
	On Local Error GoTo NOTEXPECTED

'	Definition eines Standard-Rckgabewertes
	isLeapYear() = False

	If Not isArgumentChecked(pnYear, varInteger) Then Exit Function

	isLeapYear() = ((pnYear Mod 4 = 0) And ((pnYear Mod 100 <> 0) Or (pnYear Mod 400 = 0)))
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isLeapYear V1.2, aus der Bibliothek "Tools"

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion berprft einen URL auf seine Existenz.
Public Function isFileExists(ByVal psURL As String) As Boolean
'	psURL > ein beliebiger URL

Const cstThisSub = "validation.isFileExists"
	On Local Error GoTo NOTEXPECTED

	Dim oSFA As Variant

'	Definition eines Standard-Rckgabewertes
	isFileExists() = False

	If Not isArgumentChecked(psURL, varString) Then Exit Function

	oSFA = createUnoService("com.sun.star.ucb.SimpleFileAccess")

	If oSFA.exists(psURL) Then isFileExists() = True
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isFileExists V1.1.5

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion prft, ob eine Datei schreibgeschtzt ist.
Public Function isFileReadOnly(ByVal psURL As String) As Boolean
'	psURL > ein beliebiger URL

Const cstThisSub = "validation.isFileReadOnly"
	On Local Error GoTo NOTEXPECTED

	Dim oSFA As Variant

'	Definition eines Standard-Rckgabewertes
	isFileReadOnly() = False

	If Not isArgumentChecked(psURL, varString) Then Exit Function

	oSFA = createUnoService("com.sun.star.ucb.SimpleFileAccess")

	If Not oSFA.exists(psURL) Then GoTo FILENOTFOUND

	If oSFA.isReadOnly(psURL) Then isFileReadOnly() = True 
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
	Exit Function
FILENOTFOUND:
	traceError(cstThisSub, Erl, cstFATAL, ERRFILENOTFOUND, True, psURL)
End Function 'isFileReadOnly V1.0.5, inspiriert aus der OpenOffice.org API-Dokumentation

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion berprft, ob eine OpenOffice.org-Datei bereits geffnet ist. 
Public Function isFileOpen(ByVal psURL As String) As Boolean
'	psURL > ein beliebiger URL

'	>>>TODO: Erweiterung des Parameter-Formats fr die Standardschreibweise

Const cstThisSub = "validation.isFileOpen"
	On Local Error GoTo NOTEXPECTED

	Dim oComponents As Variant
	Dim oElement As Variant

'	Definition eines Standard-Rckgabewertes
	isFileOpen() = False

	oComponents = StarDesktop.getComponents.createEnumeration
	Do While oComponents.hasMoreElements
		oElement = oComponents.nextElement()
		If hasUnoInterfaces(oElement, "com.sun.star.frame.XModel") Then
			If oElement.hasLocation() Then
				If oElement.getLocation = psURL Then
					isFileOpen() = True
					Exit Function
				End If
			End If
		End If
	Loop
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isFileOpen V1.0.5, adaptiert aus Thomas Krumbein's "OpenOffice.org Makros - Kochbuch"

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion berprft, ob eine OpenOffice.org-Datei bereits geffnet und gegebenfalls von einem anderen Benutzer zur Bearbeitung gesperrt ist.
Public Function isFileUsedBy3rdPartie(ByVal psPath As String) As Boolean
'	psPath > ein beliebiger Pfad (URL-Notation wird untersttzt)

Const cstThisSub = "validation.isFileUsedBy3rdPartie"
	On Local Error GoTo NOTEXPECTED

	Dim oSFA As Variant
	Dim sLockFile As String
	Dim sPathSeparator As String : sPathSeparator = getPathSeparator()
	Dim sLockParameters As String

'	Definition eines Standard-Rckgabewertes
	isFileUsedBy3rdPartie() = False

	oSFA = createUnoService("com.sun.star.ucb.SimpleFileAccess")

'	Umwandlung eines potenziellen URLs in den systemspezifischen Dateipfad 
	sLockFile = convertFromURL(psPath)
'	Generierung der .~lock-Datei aus dem zu ermittelndem Dateipfad
	sLockFile = getDirectoryNameOutOfPath(sLockFile, sPathSeparator) & sPathSeparator & ".~lock." & getFileNameOutOfPath(sLockFile, sPathSeparator) & "#"

'	Prfung auf Existenz der .~lock-Datei...
	If oSFA.exists(sLockFile) Then
'		... und Extrahierung der dort enthaltenen Parameter
		sLockParameters = getLastUsedLineOfFile(sLockFile)
'		falls die .~lock-Datei nicht von dem aktuellen Benutzer generiert wurde > Rckgabe des Lock-Status per TRUE
		If sLockParameters <> searchByRegExpressions(sLockParameters, _
			"^\," & cstUSERDOMAIN & "\/" & cstUSERNAME & "\," & cstUSERDOMAIN & "\,.+$") Then isFileUsedBy3rdPartie() = True
	End If
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isFileUsedBy3rdPartie V1.0

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion prft, ob ein URL ein Verzeichnis reprsentiert.
Public Function isFolder(ByVal psURL As String) As Boolean
'	psURL > ein beliebiger URL

Const cstThisSub = "validation.isFolder"
	On Local Error GoTo NOTEXPECTED

	Dim oSFA As Variant

'	Definition eines Standard-Rckgabewertes
	isFolder() = False

	If Not isArgumentChecked(psURL, varString) Then Exit Function

	oSFA = createUnoService("com.sun.star.ucb.SimpleFileAccess")

	If Not oSFA.exists(psURL) Then GoTo FILENOTFOUND

	If oSFA.isFolder(psURL) Then isFolder() = True 
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
	Exit Function
FILENOTFOUND:
	traceError(cstThisSub, Erl, cstFATAL, ERRFILENOTFOUND, True, psURL)
End Function 'isFolder V1.0, inspiriert aus der OpenOffice.org API-Dokumentation

REM --------------------------------------------------------------------------------------------------------------------------------
REM in Bearbeitung...
REM Die Funktion berprft die Gltigkeit von Argumenten anhand einer Auflistung legitimierter Objekttypen sowie auch Werten/Wertebereichen.
Public Function isArgumentChecked(ByVal pvItem As Variant, _
								  ByVal pvTypes As Variant, _
								  optional ByVal pnArgumentNumber As Integer, _
								  optional ByVal pvLimits As Variant) As Boolean  
'	pvItem			 > das Argument, welches validiert werden soll
'	pvTypes			 > legitimierte Objekttypen: als Einzelwert oder Liste
'	pnArgumentNumber > die Position des Arguments im aufrufenden Funktionskopf, beginnend von links
'	pvLimits		 > legitimierte Werte/Wertebreiche: als Einzelwert oder Liste

'	>>>TODO: pvLimits mittels String als regulrer Ausdruck realisieren?

Const cstThisSub = "validation.isArgumentChecked"
	On Local Error GoTo NOTEXPECTED

	Dim sTypes As String

'	Definition eines Standard-Rckgabewertes
	isArgumentChecked() = False

	If isMissing(pnArgumentNumber) Then pnArgumentNumber = 1

'	Prfung, ob "pvTypes" eine Liste darstellt
	If isArray(pvTypes) Then
		sTypes = joinTwice(getTypeNames(pvTypes()), ", ", getLabel("AND"))
	Else
'		... falls der einzig legitimierte Objekttyp "Boolean" ist,...
		If pvTypes = varBoolean Then
'			... und der bergebene Wert bei 0 oder 1 liegt, wird dieser fr isListed(), temporr zu "Boolean" konvertiert (automatische Typumwandlung)
			If pvItem = 0 Or pvItem = 1 Then pvItem = CBool(pvItem)
		End If

		sTypes = getTypeNames(pvTypes)
	End If

	If Not isListed(VarType(pvItem), pvTypes) Then GoTo WRONGARGUMENTTYPE

'	>>>TODO: Adaption einer Prfung fr vordefinierte Werte/Wertebereiche ber regulre Ausdrcke (z.B. Prfung eines @-Zeichens in einer Mail, etc.)
'	>>>TODO: Prfung auf - Existenz von Objekten/Variablen
'						 - korrekter Dokumenttyp?
'						 - korrekter Datentyp?
'						 - korrekte URLs/Pfade?
'						 - sind zu speichernde/zu bearbeitende Dateien schreibgeschtzt?
'						 - sind Dateien vorhanden?

	If Not isMissing(pvLimits) Then
		Dim sLimits As String

		If isArray(pvLimits) Then sLimits = joinTwice(pvLimits(), ", ", getLabel("AND")) Else sLimits = pvLimits
		If Not isListed(pvItem, pvLimits) Then GoTo WRONGARGUMENT
	End If

	isArgumentChecked() = True
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
	Exit Function
WRONGARGUMENTTYPE:
	traceError(cstThisSub, Erl, cstFATAL, ERRWRONGARGUMENTTYPE, True, Array(pnArgumentNumber, getTypeNames(VarType(pvItem)), sTypes)
	Exit Function
WRONGARGUMENT:
	traceError(cstThisSub, Erl, cstFATAL, ERRWRONGARGUMENT, True, Array(pnArgumentNumber, pvItem, sLimits))
End Function 'isArgumentChecked V1.5, inspiriert aus der Bibliothek "Access2Base"

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion berprft eine Liste auf Existenz eines bestimmten Elements.
REM Falls "pvItem" eine Zeichenfolge darstellt, kann mittels "pbCaseSensitive" auch ohne die Gross- & Kleinschreibung geprft werden.
Public Function isListed(ByVal pvItem As Variant, _
						 ByVal pvList As Variant, _
						 optional ByVal pbCaseSensitive As Boolean) As Boolean
'	pvItem			> ein beliebiges Element
'	pvList			>
'	pbCaseSensitive	> wenn FALSE, wird die Gross- und Kleinschreibung explizit NICHT beachtet (Standardwert ist TRUE)

Const cstThisSub = "validation.isListed"
	On Local Error GoTo NOTEXPECTED

'	>>>TODO: Erweiterung mit Binr-Vergleich? (s. _inList() in der Bibliothek "Access2Base")

	Dim n As Integer

'	Definition eines Standard-Rckgabewertes
	isListed() = False

	If isMissing(pbCaseSensitive) Then pbCaseSensitive = True

'	>>>TODO: Da hier die Funktion isArgumentChecked() wg. Rekursion von isListed() nicht angewendet werden kann,
'			 muss eine andere Lsung gefunden werden, um auch hier eine Validierung der Argumente zu realisieren?

'	Prfung auf fehlende Initialisierung (fehlende Definition/leere Variable)
	If VarType(pvItem) = varNull Or isNull(pvList) Then
		isListed() = False
		Exit Function
	End If

	If VarType(pvItem) = varString And Not pbCaseSensitive Then
		pvItem = UCase(pvItem)
		pvList = getUCase(pvList)
	End If

	If Not isArray(pvList) Then
		If pvItem = pvList Then isListed() = True
	Else
		For n = lBound(pvList()) To uBound(pvList())
			If pvItem = pvList(n) Then
				isListed() = True
				Exit For
			End If
		Next
	End If
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isListed V1.5

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion berprft, ob die Zeichenfolge "psBigString" mit "psPartString" beginnt.
Public Function isLeft(ByVal psBigString As String, ByVal psPartString As String, optional ByVal pbCaseSensitive As Boolean) As Boolean
'	psBigString		> eine beliebige Zeichenfolge
'	psPartString	> eine Zeichenfolge, die in "psBigString" gesucht werden soll
'	pbCaseSensitive	> wenn FALSE, wird die Gross- und Kleinschreibung explizit NICHT beachtet (Standardwert ist TRUE)

Const cstThisSub = "validation.isLeft"
	On Local Error GoTo NOTEXPECTED

	Dim nLength As Integer

'	Definition eines Standard-Rckgabewertes
	isLeft() = False

	If isMissing(pbCaseSensitive) Then pbCaseSensitive = True
	If Not pbCaseSensitive Then
		psBigString = UCase(psBigString)
		psPartString = UCase(psPartString)
	End If

	nLength = Len(psPartString)

	If Len(psBigString) >= nLength Then
		If Left(psBigString, nLength) = psPartString Then isLeft() = True
	End If
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isLeft V1.0, inspiriert aus der Bibliothek "Access2Base"

REM --------------------------------------------------------------------------------------------------------------------------------
REM Die Funktion ist ein Workarround fr die Behandlung eines internen Fehlers, wobei Ereignisse zweimal nacheinander ausgefhrt werden.
REM Bei jedem Start der Funktion wird die bergegebene Zhlervariable um Eins erhht und TRUE zurckgegeben, sobald der Maximalwert (>1) erreicht ist.
Public Function isListenerDoubleCount(ByRef pnCounter As Integer) As Boolean
'	pnCounter > eine Zhlervariable fr den jeweiligen Beobachter (kann als PRIVATE deklariert werden)

'	Beispiel des Einsatzes direkt im Beobachter: If isListenerDoubleCount(nClipboardListener) Then ... Ausfhrung einer Prozedur

Const cstThisSub = "validation.isListenerDoubleCount"
	On Local Error GoTo NOTEXPECTED

'	Definition eines Standard-Rckgabewertes
	isListenerDoubleCount() = False

	If Not isArgumentChecked(pnCounter, varInteger) Then Exit Function

	pnCounter = pnCounter + 1

	If pnCounter > 1 Then
		isListenerDoubleCount() = True
		pnCounter = 0
	End If
	Exit Function

NOTEXPECTED:
	traceError(cstThisSub, Erl, cstABORT, Err)
End Function 'isListenerDoubleCount V1.0.5
